"""A sphinx documentation configuration file.
"""

# -- Project information ---------------------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "python-livereload"

copyright = "2013, Hsiaoming Yang"

# -- General configuration -------------------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.intersphinx",
    "sphinx.ext.todo",
    "myst_parser",
    "sphinxcontrib.programoutput",
]

# -- Options for HTML output -----------------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "furo"
html_title = project

# -- Options for Autodoc --------------------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/extensions/autodoc.html#configuration

autodoc_member_order = "bysource"
autodoc_preserve_defaults = True

# -- Options for intersphinx ----------------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/extensions/intersphinx.html#configuration

intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "pypug": ("https://packaging.python.org", None),
}

# -- Options for Markdown files --------------------------------------------------------
# https://myst-parser.readthedocs.io/en/latest/sphinx/reference.html

myst_enable_extensions = [
    "colon_fence",
    "deflist",
]
myst_heading_anchors = 3



#!/usr/bin/env python

import re
from setuptools import setup


def fread(filepath):
    with open(filepath) as f:
        return f.read()


def version():
    content = fread('livereload/__init__.py')
    pattern = r"__version__ = '([0-9\.dev]*)'"
    m = re.findall(pattern, content)
    return m[0]


setup(
    name='livereload',
    version=version(),
    author='Hsiaoming Yang',
    author_email='me@lepture.com',
    url='https://github.com/lepture/python-livereload',
    packages=['livereload', 'livereload.management.commands'],
    description='Python LiveReload is an awesome tool for web developers',
    long_description_content_type='text/x-rst',
    long_description=fread('README.md'),
    entry_points={
        'console_scripts': [
            'livereload = livereload.cli:main',
        ]
    },
    install_requires=[
        'tornado',
    ],
    license='BSD',
    include_package_data=True,
    python_requires='>=3.7',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment :: Mozilla',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Compilers',
        'Topic :: Software Development :: Debuggers',
    ]
)



#!/usr/bin/python

import os
import time
import shutil
import unittest
from livereload.watcher import get_watcher_class

Watcher = get_watcher_class()

tmpdir = os.path.join(os.path.dirname(__file__), 'tmp')


class TestWatcher(unittest.TestCase):

    def setUp(self):
        if os.path.isdir(tmpdir):
            shutil.rmtree(tmpdir)
        os.mkdir(tmpdir)

    def tearDown(self):
        shutil.rmtree(tmpdir)

    def test_watch_dir(self):
        os.mkdir(os.path.join(tmpdir, '.git'))
        os.mkdir(os.path.join(tmpdir, '.hg'))
        os.mkdir(os.path.join(tmpdir, '.svn'))
        os.mkdir(os.path.join(tmpdir, '.cvs'))

        watcher = Watcher()
        watcher.watch(tmpdir)
        assert watcher.is_changed(tmpdir) is False

        # sleep 1 second so that mtime will be different
        # TODO: This doesn't seem necessary; test passes without it
        time.sleep(1)

        filepath = os.path.join(tmpdir, 'foo')

        with open(filepath, 'w') as f:
            f.write('')

        assert watcher.is_changed(tmpdir)
        assert watcher.is_changed(tmpdir) is False

        os.remove(filepath)
        assert watcher.is_changed(tmpdir)
        assert watcher.is_changed(tmpdir) is False

    def test_watch_file(self):
        watcher = Watcher()
        watcher.count = 0

        # sleep 1 second so that mtime will be different
        # TODO: This doesn't seem necessary; test passes without it
        time.sleep(1)

        filepath = os.path.join(tmpdir, 'foo')
        with open(filepath, 'w') as f:
            f.write('')

        def add_count():
            watcher.count += 1

        watcher.watch(filepath, add_count)
        assert watcher.is_changed(filepath)
        assert watcher.is_changed(filepath) is False

        # sleep 1 second so that mtime will be different
        # TODO: This doesn't seem necessary; test passes without it
        time.sleep(1)

        with open(filepath, 'w') as f:
            f.write('')

        abs_filepath = os.path.abspath(filepath)
        assert watcher.examine() == (abs_filepath, None)
        assert watcher.examine() == (None, None)
        assert watcher.count == 1

        os.remove(filepath)
        assert watcher.examine() == (abs_filepath, None)
        assert watcher.examine() == (None, None)
        assert watcher.count == 2

    def test_watch_glob(self):
        watcher = Watcher()
        watcher.watch(tmpdir + '/*')
        assert watcher.examine() == (None, None)

        with open(os.path.join(tmpdir, 'foo.pyc'), 'w') as f:
            f.write('')

        assert watcher.examine() == (None, None)

        filepath = os.path.join(tmpdir, 'foo')

        with open(filepath, 'w') as f:
            f.write('')

        abs_filepath = os.path.abspath(filepath)
        assert watcher.examine() == (abs_filepath, None)
        assert watcher.examine() == (None, None)

        os.remove(filepath)
        assert watcher.examine() == (abs_filepath, None)
        assert watcher.examine() == (None, None)

    def test_watch_ignore(self):
        watcher = Watcher()
        watcher.watch(tmpdir + '/*', ignore=lambda o: o.endswith('.ignore'))
        assert watcher.examine() == (None, None)

        with open(os.path.join(tmpdir, 'foo.ignore'), 'w') as f:
            f.write('')

        assert watcher.examine() == (None, None)

    def test_watch_multiple_dirs(self):
        first_dir = os.path.join(tmpdir, 'first')
        second_dir = os.path.join(tmpdir, 'second')

        watcher = Watcher()

        os.mkdir(first_dir)
        watcher.watch(first_dir)
        assert watcher.examine() == (None, None)

        first_path = os.path.join(first_dir, 'foo')
        with open(first_path, 'w') as f:
            f.write('')
        assert watcher.examine() == (first_path, None)
        assert watcher.examine() == (None, None)

        os.mkdir(second_dir)
        watcher.watch(second_dir)
        assert watcher.examine() == (None, None)

        second_path = os.path.join(second_dir, 'bar')
        with open(second_path, 'w') as f:
            f.write('')
        assert watcher.examine() == (second_path, None)
        assert watcher.examine() == (None, None)

        with open(first_path, 'a') as f:
            f.write('foo')
        assert watcher.examine() == (first_path, None)
        assert watcher.examine() == (None, None)

        os.remove(second_path)
        assert watcher.examine() == (second_path, None)
        assert watcher.examine() == (None, None)
